@@ -44,6 +44,7 @@ unsigned int                       log_opened; /* syslog already opened */
 struct string_list *            running_programs;
 rule *                          active_rule;
 general *                       configuration;
+int                             delay_rule;
 int                             log_level;
 void *                          libsys;   /* handle for the loaded shared library  */
 int (*libsys_scan_system_info)  (sys_info *); /* function pointer */
@@ -71,30 +72,37 @@ int main (int argc, char *argv[]) {
   /* set defaults */
   snprintf(configuration->config_file, 512,  CPUFREQD_CONFIG);
   snprintf(configuration->pidfile, 512, CPUFREQD_PIDFILE);
+  snprintf(configuration->pm_plugin, 512, DEFAULT_PMPLUGIN);
   configuration->log_level = DEFAULT_VERBOSITY; /* LOG_WARNING */
- 
+
   /* parse command line (done later than parse_config in order to 
    * be able to overwrite common parameters)
    */
   if (read_args(argc,argv)<0)
-    return(-1);
+    return 1;
   
   /* check cpufreq interface */
   if (find_cpufreq_interface(configuration)<0) {
     cp_log(LOG_CRIT, "Unable to find a cpufreq interface, please ensure to have cpufreq enabled in the running kernel.\nExiting.\n");
-    return(-1);
+    return 1;
+  }
+
+  /* get number of cpus */
+  if (get_cpu_num(configuration)<0) {
+    cp_log(LOG_CRIT, "Unable to find any CPUs, please check your procfs.\nExiting.\n");
+    return 1;
   }
  
   /* read configuration */
   if (parse_config(configuration) < 0) {
-    cp_log(LOG_CRIT, "Unable to parse configuration file %s.\nExiting.\n", configuration->config_file);
-    return(-1);
+    cp_log(LOG_CRIT, "Failed to parse configuration file %s.\nExiting.\n", configuration->config_file);
+    return 1;
   }
   
   /* check perms */
   if (geteuid() != 0) {
     cp_log(LOG_CRIT, "%s: must be run as root.\n", argv[0]);
-    return(-1);
+    return 1;
   }
 
   cp_log(LOG_NOTICE, "Starting operations.\n");
@@ -118,19 +126,19 @@ int main (int argc, char *argv[]) {
   /* open libraries */
   if (load_libraries() < 0) {
     cp_log(LOG_CRIT, "Cannot open shared libraries.\nExiting.\n");
-    return -1;
+    return 1;
   }
 
   /* go background and setup stderr, stdout and stdin */
   if (!configuration->no_daemon && daemonize()<0) {
     cp_log(LOG_CRIT, "Unable to go to background.\nExiting.\n");
-    return(-1);
+    return 1;
   }
  
   /* write pidfile */
   if (write_cpufreqd_pid() < 0) {
     cp_log(LOG_CRIT, "Cannot write pidfile %s.\nExiting.\n", configuration->pidfile);
-    return -1;
+    return 1;
   }
   cp_log(LOG_INFO, "pid: %d\n", getpid());
 
@@ -140,14 +148,15 @@ int main (int argc, char *argv[]) {
 
   if (clear_cpufreqd_pid() < 0) {
     cp_log(LOG_ERR, "Unable to remove %s.\n", configuration->pidfile);
-    return -1;
+    return 1;
   }
   
   /* free configuration */
   free_config(configuration);
   
+  /* TODO: this call to close log shound not sit here... */
   closelog();
-  return(0);
+  return 0;
 }
 
 /* int load_libraries(void)
@@ -337,14 +346,14 @@ int main_loop(void) {
     /* scan_system_info */
     if ((*libsys_scan_system_info)(_system)<0) {
       cp_log(LOG_ERR, "main_loop(): Can't get system info. Exiting.\n");
-      return(-1);
+      return -1;
     }
     
     /* get_cpu */
     _system->cpu_percent = (*libsys_get_cpu)();
     if (_system->cpu_percent == -1) {
-      cp_log(LOG_INFO, "main_loop(): Can't get CPU usage. Exiting.\n");
-      return(-1);
+      cp_log(LOG_ERR, "main_loop(): Can't get CPU usage. Exiting.\n");
+      return -1;
     }
 
     /* read running programs */
@@ -358,15 +367,14 @@ int main_loop(void) {
 
         /* find a valid rule */
         r_temp = find_valid_rule(_system, configuration->rules);
-        /* check if the new rule is different from the currently active one */
 
-/**
- *  BUG: TODO - if (r_temp==NULL)
- * **/
+        /* if no valid rule has been found leave system untouched and go on */
         if (r_temp == NULL) {
+          cp_log(LOG_WARNING, "main_loop(): no valid rule found check your config file against current system state\n");
           continue;
         }
         
+        /* check if the new rule is different from the currently active one */
         if (strcmp(r_temp->name, active_rule->name)!=0) {
           /* set profile_changed to avoid writing a new policy if not necessary */
           profile_changed = strcmp(r_temp->prof->name, active_rule->prof->name);
@@ -376,11 +384,15 @@ int main_loop(void) {
           /* no need to write it if it is the same profile as before */
           if (profile_changed) {
             if (active_rule && set_policy(active_rule->prof) < 0) {
-              return(-1);
+              return -1;
             }
             cp_log(LOG_NOTICE, "main_loop(): profile set \"%s\" for rule \"%s\".\n",
                 active_rule->prof->name, active_rule->name);
           } /* end if profile changed */
+          else {
+            cp_log(LOG_NOTICE, "main_loop(): profile %s already set, skipping.\n", 
+                active_rule->prof->name);
+          }
         } /* end if rule changed */
       } /* end if rule no more valid */
     } else {
@@ -388,13 +400,21 @@ int main_loop(void) {
       
       /* find a valid rule */
       active_rule = find_valid_rule(_system, configuration->rules);
-
+      
+      /* if no valid rule has been found leave system untouched and go on */
+      if (active_rule==NULL) {
+        cp_log(LOG_WARNING, "main_loop(): no valid rule found check your config file against current system state\n");
+        continue;
+      }
+      
       /* set the associated policy */
-      if (active_rule && set_policy(active_rule->prof) < 0)
-        return(-1);
-
+      if (set_policy(active_rule->prof) < 0) {
+        cp_log(LOG_ERR, "main_loop(): cannot set current policy. Exinting\n");
+        return -1;
+      }
+    
       cp_log(LOG_NOTICE, "main_loop(): profile set \"%s\" for rule \"%s\".\n",
-          active_rule->prof->name, active_rule->name);
+             active_rule->prof->name, active_rule->name);
     }
     
     if (force_reinit)
@@ -403,7 +423,7 @@ int main_loop(void) {
       sleep(configuration->poll_interval);
   } /* end loop */
 
-  return(0);
+  return 0;
 }
 
 /* int write_cpufreqd_pid(void)
@@ -486,40 +506,48 @@ int clear_cpufreqd_pid(void) {
  */
 int set_policy(profile *prof) {
   FILE *fp;
-
+  char *path = (char *)malloc(256);
+  
   if (!prof)
-    return(-1);
+    return -1;
 
-  /* /sys/devices/sys/cpu0 */
+  /* /sys/devices/system/cpu* */
   if (configuration->has_sysfs) {
-    /* governor */
-    fp = fopen(CPUFREQ_SYSFS_INTERFACE_POLICY, "w");
-    if (!fp) {
-      cp_log(LOG_ERR, "set_policy(): %s: %s\n", CPUFREQ_SYSFS_INTERFACE_POLICY, strerror(errno));
-      return(-1);
-    }
-    fprintf(fp, "%s", prof->policy_name);
-    fclose(fp);
+    int cpu;
 
-    /* max speed */
-    fp = fopen(CPUFREQ_SYSFS_INTERFACE_MAX, "w");
-    if (!fp) {
-      cp_log(LOG_ERR, "set_policy(): %s: %s\n", CPUFREQ_SYSFS_INTERFACE_MAX, strerror(errno));
-      return(-1);
+    for(cpu=0; cpu < configuration->cpu_num; cpu++) {
+      /* governor */
+      snprintf(path, 256, CPUFREQ_SYSFS_INTERFACE_POLICY, cpu);
+      fp = fopen(path, "w");
+      if (!fp) {
+        cp_log(LOG_ERR, "set_policy(): %s: %s\n", path, strerror(errno));
+        return -1;
+      }
+      fprintf(fp, "%s", prof->policy_name);
+      fclose(fp);
+
+      /* max speed */
+      snprintf(path, 256, CPUFREQ_SYSFS_INTERFACE_MAX, cpu);
+      fp = fopen(path, "w");
+      if (!fp) {
+        cp_log(LOG_ERR, "set_policy(): %s: %s\n", path, strerror(errno));
+        return -1;
+      }
+      fprintf(fp, "%li", prof->max_freq);
+      fclose(fp);
+
+      /* min speed */
+      snprintf(path, 256, CPUFREQ_SYSFS_INTERFACE_MIN, cpu);
+      fp = fopen(path, "w");
+      if (!fp) {
+        cp_log(LOG_ERR, "set_policy(): %s: %s\n", path, strerror(errno));
+        return -1;
+      }
+      fprintf(fp, "%li", prof->min_freq);
+      fclose(fp);
+
+      cp_log(LOG_NOTICE, "set_policy(): %s CPU%i %li %li.\n", prof->policy_name, cpu, prof->min_freq, prof->max_freq);
     }
-    fprintf(fp, "%li", prof->max_freq);
-    fclose(fp);
-                
-    /* min speed */
-    fp = fopen(CPUFREQ_SYSFS_INTERFACE_MIN, "w");
-    if (!fp) {
-      cp_log(LOG_ERR, "set_policy(): %s: %s\n", CPUFREQ_SYSFS_INTERFACE_MIN, strerror(errno));
-      return(-1);
-    }
-    fprintf(fp, "%li", prof->min_freq);
-    fclose(fp);
-    
-    cp_log(LOG_NOTICE, "set_policy(): %s %li %li.\n", prof->policy_name, prof->min_freq, prof->max_freq);
 
   } else {
 
@@ -527,24 +555,23 @@ int set_policy(profile *prof) {
     fp = fopen(CPUFREQ_PROC_INTERFACE, "w");
     if (!fp) {
       cp_log(LOG_ERR, "set_policy(): %s: %s\n", CPUFREQ_PROC_INTERFACE, strerror(errno));
-      return(-1);
+      return -1;
     }
 
-    cp_log(LOG_NOTICE, "set_policy(): 0%c%li%c%li%c%s - profile name: %s.\n",
-                     prof->sep, prof->min_freq, 
+    cp_log(LOG_NOTICE, "set_policy(): %li%c%li%c%s - profile name: %s.\n",
+                     prof->min_freq, 
                      prof->sep, prof->max_freq,
                      prof->sep, prof->policy_name, 
                      prof->name);
 
-    /* TODO: multiple cpu support */
-    fprintf(fp, "0%c%li%c%li%c%s", 
-                     prof->sep, prof->min_freq, 
+    fprintf(fp, "%li%c%li%c%s", 
+                     prof->min_freq, 
                      prof->sep, prof->max_freq,
                      prof->sep, prof->policy_name);
 
     fclose(fp);
   }
-  return(0);
+  return 0;
 }
 
 
@@ -603,6 +630,45 @@ rule *find_valid_rule(const sys_info *s, rule *r) {
   return NULL;
 }
 
+/*  inline int intersects (const struct interval *i, const int *val) 
+ *
+ *  Check if rule's cpu_interval intersects actual cpu usage
+ *  weigthing nice_time based on rule's settings
+ *
+ *  Returns 1 if the interval intersects 0 otherwise
+ */
+/*
+int intersects (const struct interval *i, const int *val) {
+  return (*val >= i->min && *val <= i->max);
+}
+*/
+
+/*
+ *
+ */
+/*
+int match_cpu_usage (sys_info *si, cpu_interval *cpu, int nice_scale) {
+  int delta_activity = 0, weighted_activity = 0;
+  
+  weighted_activity = si->cur_sys_activity + si->cur_nice_activity / nice_scale;
+  delta_activity = weighted_activity - si->old_weighted_activity;
+  old_weighted_activity = weighted_activity;
+
+  cp_log(LOG_DEBUG,
+         "get_cpu(): CPU delta_activity=%d delta_time=%d weighted_activity=%d.\n",
+         delta_activity, delta_time, weighted_activity);
+
+  weighted_activity = c_user + c_nice / 3 + c_sys;
+  
+  if ( delta_activity > delta_time || delta_time <= 0) {
+    perc = 100;
+  } else {
+    perc = delta_activity * 100 / delta_time;
+  }
+  cp_log(LOG_INFO, "get_cpu(): CPU usage = %d.\n", perc);
+  return 0;
+}
+*/
 
 /* int get_rule_hits(const sys_info *si, const rule *r, int necessary_hits)
  * 
@@ -614,7 +680,7 @@ rule *find_valid_rule(const sys_info *s, rule *r) {
  *
  */
 int get_rule_hits(const sys_info *si, const rule *r) {
-  int hits = 0, debug = 0;
+  int hits = 0, debug_count = 0;
   
   /* check battery */
   if (si->battery_percent >= r->bat->min && si->battery_percent <= r->bat->max) { 
@@ -659,9 +725,9 @@ int get_rule_hits(const sys_info *si, const rule *r) {
         pr_config = pr_config->next;
       } /* end configured programs loop */
       pr_iter = pr_iter->next;
-      debug++;
+      debug_count++;
     } /* end full program list loop */
-    cp_log(LOG_DEBUG, "get_rule_hits(): checked %d processes\n", debug);
+    cp_log(LOG_DEBUG, "get_rule_hits(): checked %d processes\n", debug_count);
   }
   cp_log(LOG_DEBUG, "get_rule_hits(): hits for %s: %d\n", r->name, hits);
   return hits;
